@@ -30,10 +30,13 @@ fn tick_offset_map(axis: &axis::Axis, face_width: u32) -> HashMap<i32, f64> {
 /// the total scale of the axis
 /// and the number of face cells to work with,
 /// create a mapping of cell offset to bin bound
-fn bound_cell_offsets(hist: &histogram::Histogram, face_width: u32) -> Vec<i32> {
+fn bound_cell_offsets(hist: &histogram::Histogram,
+                      x_axis: &axis::Axis,
+                      face_width: u32)
+                      -> Vec<i32> {
     hist.bin_bounds
         .iter()
-        .map(|&bound| value_to_axis_cell_offset(bound, &hist.x_axis, face_width))
+        .map(|&bound| value_to_axis_cell_offset(bound, &x_axis, face_width))
         .collect()
 }
 
@@ -119,9 +122,7 @@ fn create_x_axis_labels(x_tick_map: &HashMap<i32, f64>) -> Vec<XAxisLabel> {
     ls
 }
 
-fn render_y_axis_strings(y_axis: &axis::Axis,
-                         face_height: u32)
-                         -> (Vec<String>, Vec<String>, Vec<String>, i32) {
+pub fn render_y_axis_strings(y_axis: &axis::Axis, face_height: u32) -> (String, i32) {
     // Get the strings and offsets we'll use for the y-axis
     let y_tick_map = tick_offset_map(&y_axis, face_height);
 
@@ -154,10 +155,26 @@ fn render_y_axis_strings(y_axis: &axis::Axis,
         .map(|s| s.to_string())
         .collect();
 
-    (y_label_strings, y_tick_strings, y_axis_line_strings, longest_y_label_width as i32)
+    let iter = y_label_strings.iter()
+        .zip(y_tick_strings.iter())
+        .zip(y_axis_line_strings.iter())
+        .map(|((x, y), z)| (x, y, z));
+
+    let axis_string: Vec<String> = iter.rev()
+        .map(|(l, t, a)| {
+            format!("{:>num_width$}{}{}",
+                    l,
+                    t,
+                    a,
+                    num_width = longest_y_label_width)
+        })
+        .collect();
+    let axis_string = axis_string.join("\n");
+
+    (axis_string, longest_y_label_width as i32)
 }
 
-fn render_x_axis_strings(x_axis: &axis::Axis, face_width: u32) -> (String, String, String, i32) {
+pub fn render_x_axis_strings(x_axis: &axis::Axis, face_width: u32) -> (String, i32) {
     // Get the strings and offsets we'll use for the x-axis
     let x_tick_map = tick_offset_map(x_axis, face_width as u32);
 
@@ -200,15 +217,37 @@ fn render_x_axis_strings(x_axis: &axis::Axis, face_width: u32) -> (String, Strin
         .chain(std::iter::repeat('-').take(face_width as usize))
         .collect();
 
-    (x_axis_label_string, x_axis_tick_string, x_axis_line_string, start_offset)
+    let x_axis_string = if start_offset.is_positive() {
+        let padding = (0..start_offset).map(|_| " ").collect::<String>();
+        format!("{}\n{}\n{}{}",
+                x_axis_line_string,
+                x_axis_tick_string,
+                padding,
+                x_axis_label_string)
+    } else {
+        let padding = (0..start_offset.wrapping_neg()).map(|_| " ").collect::<String>();
+        format!("{}{}\n{}{}\n{}",
+                padding,
+                x_axis_line_string,
+                padding,
+                x_axis_tick_string,
+                x_axis_label_string)
+    };
+
+    (x_axis_string, start_offset)
 }
 
 /// Given a histogram,
 /// the x ands y-axes
 /// and the face height and width,
 /// create the strings to be drawn as the face
-fn render_face_bars(h: &histogram::Histogram, face_width: u32, face_height: u32) -> Vec<String> {
-    let bound_cells = bound_cell_offsets(&h, face_width);
+pub fn render_face_bars(h: &histogram::Histogram,
+                        x_axis: &axis::Axis,
+                        y_axis: &axis::Axis,
+                        face_width: u32,
+                        face_height: u32)
+                        -> String {
+    let bound_cells = bound_cell_offsets(&h, &x_axis, face_width);
 
     let cell_bins = bins_for_cells(&bound_cells, face_width);
 
@@ -217,7 +256,7 @@ fn render_face_bars(h: &histogram::Histogram, face_width: u32, face_height: u32)
         .map(|&bin| match bin {
             None => 0,
             Some(b) => {
-                value_to_axis_cell_offset(h.bin_counts[b as usize] as f64, &h.y_axis, face_height)
+                value_to_axis_cell_offset(h.bin_counts[b as usize] as f64, &y_axis, face_height)
             }
         })
         .collect();
@@ -268,183 +307,120 @@ fn render_face_bars(h: &histogram::Histogram, face_width: u32, face_height: u32)
         }
         face_strings.push(line_string);
     }
-    face_strings
+    let face_strings: Vec<String> = face_strings.iter().rev().map(|s| s.clone()).collect();
+    face_strings.join("\n")
 }
 
 /// Given a scatter plot,
 /// the x ands y-axes
 /// and the face height and width,
 /// create the strings to be drawn as the face
-fn render_face_points(s: &scatter::Scatter, face_width: u32, face_height: u32) -> Vec<String> {
+pub fn render_face_points(s: &scatter::Scatter,
+                          x_axis: &axis::Axis,
+                          y_axis: &axis::Axis,
+                          face_width: u32,
+                          face_height: u32,
+                          style: &scatter::Style)
+                          -> String {
 
     let points: Vec<_> = s.data
         .iter()
         .map(|&(x, y)| {
-            (value_to_axis_cell_offset(x, &s.x_axis, face_width),
-             value_to_axis_cell_offset(y, &s.y_axis, face_height))
+            (value_to_axis_cell_offset(x, &x_axis, face_width),
+             value_to_axis_cell_offset(y, &y_axis, face_height))
         })
         .collect();
 
+    let marker = match style.get_marker() {
+        scatter::Marker::Circle => '●',
+        scatter::Marker::Square => '■',
+    };
+
     let mut face_strings: Vec<String> = vec![];
     for line in 1..face_height + 1 {
         let mut line_string = String::new();
         for column in 1..face_width as usize + 1 {
             line_string.push(if points.contains(&(column as i32, line as i32)) {
-                'o'
+                marker
             } else {
                 ' '
             });
         }
         face_strings.push(line_string);
     }
-    face_strings
+    let face_strings: Vec<String> = face_strings.iter().rev().map(|s| s.clone()).collect();
+    face_strings.join("\n")
 }
 
-pub fn draw_histogram(h: &histogram::Histogram) {
-    // The face is the actual area of the graph with data on it, excluding axes and labels
-    let face_width = 90;
-    let face_height = 30u32;
+/// Given two 'rectangular' strings, overlay the second on the first offset by `x` and `y`
+pub fn overlay(under: &str, over: &str, x: i32, y: i32) -> String {
+    let split_under: Vec<_> = under.split('\n').collect();
+    let under_width = split_under.iter().map(|s| s.len()).max().unwrap();
+    let under_height = split_under.len();
 
-    ////////////
-    // Y Axis //
-    ////////////
+    let split_over: Vec<String> = over.split('\n').map(|s| s.to_string()).collect();
+    let over_width = split_over.iter().map(|s| s.len()).max().unwrap();
 
-    let (y_label_strings, y_tick_strings, y_axis_line_strings, longest_y_label_width) =
-        render_y_axis_strings(&h.y_axis, face_height);
+    // Take `over` and pad it so that it matches `under`'s dimensions
 
-    ////////////
-    // X Axis //
-    ////////////
+    // Trim/add lines at beginning
+    let split_over: Vec<String> = if y.is_negative() {
+        split_over.iter().skip(y.abs() as usize).map(|s| s.clone()).collect()
+    } else if y.is_positive() {
+        (0..y)
+            .map(|_| (0..over_width).map(|_| ' ').collect())
+            .chain(split_over.iter().map(|s| s.to_string()))
+            .collect()
+    } else {
+        split_over
+    };
 
-    let (x_axis_label_string, x_axis_tick_string, x_axis_line_string, start_offset) =
-        render_x_axis_strings(&h.x_axis, face_width);
+    // Trim/add chars at beginning
+    let split_over: Vec<String> = if x.is_negative() {
+        split_over.iter().map(|l| l.chars().skip(x.abs() as usize).collect()).collect()
+    } else if x.is_positive() {
+        split_over.iter().map(|s| (0..x).map(|_| ' ').chain(s.chars()).collect()).collect()
+    } else {
+        split_over
+    };
 
-    //////////
-    // Face //
-    //////////
+    // pad out end of vector
+    let over_width = split_over.iter().map(|s| s.len()).max().unwrap();
+    let over_height = split_over.len();
+    let lines_deficit = under_height as i32 - over_height as i32;
+    let split_over: Vec<String> = if lines_deficit.is_positive() {
+        let new_lines: Vec<String> =
+            (0..lines_deficit).map(|_| (0..over_width).map(|_| ' ').collect::<String>()).collect();
+        let mut temp = split_over.clone();
+        for new_line in new_lines {
+            temp.push(new_line);
+        }
+        temp
+    } else {
+        split_over
+    };
 
-    let face_strings = render_face_bars(&h, face_width as u32, face_height);
+    // pad out end of each line
+    let line_width_deficit = under_width as i32 - over_width as i32;
+    let split_over: Vec<String> = if line_width_deficit.is_positive() {
+        split_over.iter()
+            .map(|l| l.chars().chain((0..line_width_deficit).map(|_| ' ')).collect())
+            .collect()
+    } else {
+        split_over
+    };
 
-    /////////////
-    // Drawing //
-    /////////////
-
-    let left_gutter_width = std::cmp::max(longest_y_label_width as i32 + 1,
-                                          start_offset.wrapping_neg()) as
-                            usize;
-
-    // Go, line-by-line printing the y-axis and the face
-    for line in 0..face_height {
-        let cell_position = face_height - line;
-        let axis_label = &y_label_strings[cell_position as usize];
-        let axis_tick = &y_tick_strings[cell_position as usize];
-        let axis_line = &y_axis_line_strings[cell_position as usize];
-        let mut axis_label_and_tick = String::new();
-        axis_label_and_tick.push_str(axis_label);
-        axis_label_and_tick.push_str(axis_tick);
-        let face_line = &face_strings[(cell_position as usize) - 1];
-        println!("{:>left_gutter_width$}{}{}",
-                 axis_label_and_tick,
-                 axis_line,
-                 face_line,
-                 left_gutter_width = left_gutter_width);
+    // Now that the dimensions match, overlay them
+    let mut out: Vec<String> = vec![];
+    for (l, ol) in split_under.iter().zip(split_over.iter()) {
+        let mut new_line = "".to_string();
+        for (c, oc) in l.chars().zip(ol.chars()) {
+            new_line.push(if oc == ' ' { c } else { oc });
+        }
+        out.push(new_line);
     }
 
-    // Avoid duplicating the '+'. TODO combine them cleverly
-    let reduced_x_axis_line = x_axis_line_string[1..].to_string();
-    // Print the x-axis line and the zero-point of the y-axis
-    println!("{:>left_gutter_width$}{}{}",
-             y_label_strings[0],
-             y_axis_line_strings[0],
-             reduced_x_axis_line,
-             left_gutter_width = left_gutter_width);
-
-    // Print the x-axis ticks
-    println!("{:>left_gutter_width$}{:-<plot_width$}",
-             "",
-             x_axis_tick_string,
-             left_gutter_width = left_gutter_width,
-             plot_width = face_width as usize + 1);
-
-    // Print the x-axis labels
-    println!("{:>start_offset$}{: <plot_width$}",
-             "",
-             x_axis_label_string,
-             start_offset = (left_gutter_width as i32 + start_offset) as usize,
-             plot_width = (face_width as i32 + 1 + start_offset) as usize);
-}
-
-pub fn draw_scatter(s: &scatter::Scatter) {
-    let face_width = 90;
-    let face_height = 30u32;
-
-    ////////////
-    // Y Axis //
-    ////////////
-
-    let (y_label_strings, y_tick_strings, y_axis_line_strings, longest_y_label_width) =
-        render_y_axis_strings(&s.y_axis, face_height);
-
-    ////////////
-    // X Axis //
-    ////////////
-
-    let (x_axis_label_string, x_axis_tick_string, x_axis_line_string, start_offset) =
-        render_x_axis_strings(&s.x_axis, face_width);
-
-    //////////
-    // Face //
-    //////////
-
-    let face_strings = render_face_points(&s, face_width as u32, face_height);
-
-    /////////////
-    // Drawing //
-    /////////////
-
-    let left_gutter_width = std::cmp::max(longest_y_label_width as i32 + 1,
-                                          start_offset.wrapping_neg()) as
-                            usize;
-
-    // Go, line-by-line printing the y-axis and the face
-    for line in 0..face_height {
-        let cell_position = face_height - line;
-        let axis_label = &y_label_strings[cell_position as usize];
-        let axis_tick = &y_tick_strings[cell_position as usize];
-        let axis_line = &y_axis_line_strings[cell_position as usize];
-        let mut axis_label_and_tick = String::new();
-        axis_label_and_tick.push_str(axis_label);
-        axis_label_and_tick.push_str(axis_tick);
-        let face_line = &face_strings[(cell_position as usize) - 1];
-        println!("{:>left_gutter_width$}{}{}",
-                 axis_label_and_tick,
-                 axis_line,
-                 face_line,
-                 left_gutter_width = left_gutter_width);
-    }
-
-    // Avoid duplicating the '+'. TODO combine them cleverly
-    let reduced_x_axis_line = x_axis_line_string[1..].to_string();
-    // Print the x-axis line and the zero-point of the y-axis
-    println!("{:>left_gutter_width$}{}{}",
-             y_label_strings[0],
-             y_axis_line_strings[0],
-             reduced_x_axis_line,
-             left_gutter_width = left_gutter_width);
-
-    // Print the x-axis ticks
-    println!("{:>left_gutter_width$}{:-<plot_width$}",
-             "",
-             x_axis_tick_string,
-             left_gutter_width = left_gutter_width,
-             plot_width = face_width as usize + 1);
-
-    // Print the x-axis labels
-    println!("{:>start_offset$}{: <plot_width$}",
-             "",
-             x_axis_label_string,
-             start_offset = (left_gutter_width as i32 + start_offset) as usize,
-             plot_width = (face_width as i32 + 1 + start_offset) as usize);
+    out.join("\n")
 }
 
 #[cfg(test)]
@@ -524,4 +500,113 @@ mod tests {
         assert!(l.footprint() % 2 != 0);
         assert_eq!(l.start_offset(), -1);
     }
+
+    #[test]
+    fn test_render_y_axis_strings() {
+        let y_axis = axis::Axis::new(0.0, 10.0);
+
+        let (y_axis_string, longest_y_label_width) = render_y_axis_strings(&y_axis, 10);
+
+        assert!(y_axis_string.contains(&"0".to_string()));
+        assert!(y_axis_string.contains(&"6".to_string()));
+        assert!(y_axis_string.contains(&"10".to_string()));
+        assert_eq!(longest_y_label_width, 2);
+    }
+
+    #[test]
+    fn test_render_x_axis_strings() {
+        let x_axis = axis::Axis::new(0.0, 10.0);
+
+        let (x_axis_string, start_offset) = render_x_axis_strings(&x_axis, 20);
+
+        assert!(x_axis_string.contains("0 "));
+        assert!(x_axis_string.contains(" 6 "));
+        assert!(x_axis_string.contains(" 10"));
+        assert_eq!(x_axis_string.chars().filter(|&c| c == '|').count(), 6);
+        assert_eq!(start_offset, 0);
+    }
+
+    #[test]
+    fn test_render_face_bars() {
+        let data = vec![0.3, 0.5, 6.4, 5.3, 3.6, 3.6, 3.5, 7.5, 4.0];
+        let h = histogram::Histogram::from_vec(&data, 10);
+        let x_axis = axis::Axis::new(0.3, 7.5);
+        let y_axis = axis::Axis::new(0., 3.);
+        let strings = render_face_bars(&h, &x_axis, &y_axis, 20, 10);
+        assert_eq!(strings.lines().count(), 10);
+        assert!(strings.lines().all(|s| s.chars().count() == 20));
+
+        let comp = vec!["       ---          ",
+                        "       | |          ",
+                        "       | |          ",
+                        "--     | |          ",
+                        " |     | |          ",
+                        " |     | |          ",
+                        " |     | |          ",
+                        " |     | |---- -----",
+                        " |     | | | | | | |",
+                        " |     | | | | | | |"]
+            .join("\n");
+
+        assert_eq!(&strings, &comp);
+    }
+
+    #[test]
+    fn test_render_face_points() {
+        let data = vec![(-3.0, 2.3), (-1.6, 5.3), (0.3, 0.7), (4.3, -1.4), (6.4, 4.3), (8.5, 3.7)];
+        let s = scatter::Scatter::from_vec(&data);
+        let x_axis = axis::Axis::new(-3.575, 9.075);
+        let y_axis = axis::Axis::new(-1.735, 5.635);
+        let style = scatter::Style::new();
+        let strings = render_face_points(&s, &x_axis, &y_axis, 20, 10, &style);
+        assert_eq!(strings.lines().count(), 10);
+        assert!(strings.lines().all(|s| s.chars().count() == 20));
+
+        let comp = vec!["  ●                 ",
+                        "                    ",
+                        "               ●    ",
+                        "                  ● ",
+                        "                    ",
+                        "●                   ",
+                        "                    ",
+                        "     ●              ",
+                        "                    ",
+                        "                    "]
+            .join("\n");
+
+        assert_eq!(&strings, &comp);
+    }
+
+    #[test]
+    fn test_overlay() {
+        let a = " ooo ";
+        let b = "  #  ";
+        let r = " o#o ";
+        assert_eq!(overlay(a, b, 0, 0), r);
+
+        let a = " o o o o o o o o o o ";
+        let b = "# # # # #";
+        let r = " o#o#o#o#o#o o o o o ";
+        assert_eq!(overlay(a, b, 2, 0), r);
+
+        let a = "     \n   o \n o  o\nooooo\no o o";
+        let b = "  #  \n   # \n     \n  ## \n   ##";
+        let r = "  #  \n   # \n o  o\noo##o\no o##";
+        assert_eq!(overlay(a, b, 0, 0), r);
+
+        let a = "     \n   o \n o  o\nooooo\no o o";
+        let b = "  #\n## ";
+        let r = "     \n   o \n o #o\no##oo\no o o";
+        assert_eq!(overlay(a, b, 1, 2), r);
+
+        let a = "     \n   o \n o  o\nooooo\no o o";
+        let b = "###\n###\n###";
+        let r = "##   \n## o \n o  o\nooooo\no o o";
+        assert_eq!(overlay(a, b, -1, -1), r);
+
+        let a = "oo\noo";
+        let b = "    \n  # \n #  \n    ";
+        let r = "o#\n#o";
+        assert_eq!(overlay(a, b, -1, -1), r);
+    }
 }
